/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: dos_i.h,v 1.5 1993/05/15 15:34:36 mura Exp $
 */

#ifndef __sys_dos_i_h__
#define __sys_dos_i_h__

#if !defined (__GNUC__) && !defined (__GNUG__)
#error You lose. This file can be compiled only by GNU-C compiler.
#endif

/*
 * The following functions cannot be inline-expanded.
 * Please use library version instead.
 *
 *	_dos_importlnenv
 *	_dos_lfiles
 *	_dos_link
 *	_dos_readlink
 *	_dos_symlink
 *	_dos_unlink
 */

#ifndef __cdecl_h__
#include <cdecl.h>
#endif

EXT (int_ *_dos_importlnenv (void_));
EXT (int_ _dos_lfiles (struct _filbuf *, const char *, int_));
EXT (int_ _dos_link (const char *, const char *));
EXT (int_ _dos_readlink (const char *, char *, int_));
EXT (int_ _dos_symlink (const char *, const char *));
EXT (int_ _dos_unlink (const char *));

static __inline void __volatile _dos_exit (void_)
{
    asm volatile ("dc.w EXIT");
}

static __inline int_ _dos_getchar (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w GETCHAR"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline void _dos_putchar (int_ code)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "dc.w PUTCHAR\n\t"
		  "addq.w #2,sp"
		  :/* no outputs */
		  :"ri" ((short) code));
}

static __inline int_ _dos_cominp (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w COMINP"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline void _dos_comout (int_ code)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "dc.w COMOUT\n\t"
		  "addq.w #2,sp"
		  :/* no outputs */
		  :"ri" ((short) code));
}

static __inline void _dos_prnout (int_ code)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "dc.w PRNOUT\n\t"
		  "addq.w #2,sp"
		  :/* no outputs */
		  :"ri" ((short) code)
		  :"d0");
}

static __inline int_ _dos_inpout (int_ code)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w INPOUT\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) code));

    return reg_d0;
}

static __inline int_ _dos_inkey (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w INKEY"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_getc (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w GETC"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline void _dos_print (const char *string)
{
    asm volatile ("move.l %0,-(sp)\n\t"
		  "dc.w PRINT\n\t"
		  "addq.w #4,sp"
		  :/* no outputs */
		  :"g" ((int_) string)
		  :"d0");
}

static __inline int_ _dos_gets (struct _inpptr *ptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "dc.w PRINT\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0), "=m" (*ptr)
		  :"g" ((int_) ptr));

    return reg_d0;
}

static __inline int_ _dos_keysns (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w KEYSNS"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_kflushgp (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #1,-(sp)\n\t"
		  "dc.w KFLUSH\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_kflushio (int_ code)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #$06,-(sp)\n\t"
		  "dc.w KFLUSH\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) code));

    return reg_d0;
}

static __inline int_ _dos_kflushin (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #$07,-(sp)\n\t"
		  "dc.w KFLUSH\n\t"
		  "addq.w #2,sp"
		  :"=d" ((short) reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_kflushgc (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #$08,-(sp)\n\t"
		  "dc.w KFLUSH\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_kflushgs (struct _inpptr *ptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.w #$0a,-(sp)\n\t"
		  "dc.w KFLUSH\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0), "=m" (*ptr)
		  :"g" ((int_) ptr));

    return reg_d0;
}

static __inline void _dos_fflush (void_)
{
    asm volatile ("dc.w FFLUSH"
		  :/* no outputs */
		  :/* no inputs */);
}

static __inline int_ _dos_chgdrv (int_ drv)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w CHGDRV\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) drv));

    return reg_d0;
}

static __inline int_ _dos_drvctrl (int_ mode, int_ drv)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,%0\n\t"
		  "lsl.w  #8,%0\n\t"
		  "move.b %2,%0\n\t"
		  "move.w %0,-(sp)\n\t"
		  "dc.w DRVCTRL\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) mode), "dm" ((char) drv));

    return reg_d0;
}

static __inline int_ _dos_consns (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w CONSNS"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_prnsns (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w PRNSNS"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_cinsns (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w COUTSNS"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_fatchk (const char *file, unsigned short *bufptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w FATCHK\n\t"
		  "addq.w  #8,sp"
		  :"=d" (reg_d0), "=m" (*bufptr)
		  :"g" ((int_) bufptr), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_fatchk2 (const char *file,
				   unsigned short *bufptr, int_ length)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %2,-(sp)\n\t"
		  "bset.l #31,%3\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.l %4,-(sp)\n\t"
		  "dc.w FATCHK\n\t"
		  "lea.l 10(sp),sp"
		  :"=d" (reg_d0), "=m" (*bufptr)
		  :"ri" ((short) length), "dm" ((int_) bufptr), "g" ((int_) file));
           return reg_d0;
}

static __inline int_ _dos_hendspmo (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("clr.w -(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_hendspmp (int_ pos, const char *mesptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #1,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) mesptr), "ri" ((short) pos));

    return reg_d0;
}

static __inline int_ _dos_hendspmr (int_ pos, const char *mesptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #2,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) mesptr), "ri" ((short) pos));

    return reg_d0;
}

static __inline void _dos_hendspmc (void_)
{
    asm volatile ("move.w #3,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #2,sp"
		  :/* no outputs */
		  :/* no inputs */);
}

static __inline int_ _dos_hendspio (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #4,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_hendspip (int_ pos, const char *mesptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #5,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) mesptr), "ri" ((short) pos));

    return reg_d0;
}

static __inline int_ _dos_hendspir (int_ pos, const char *mesptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #6,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) mesptr), "ri" ((short) pos));

    return reg_d0;
}

static __inline void _dos_hendspic (int_ pos)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "move.w #7,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #4,sp"
		  :/* no outputs */
		  :"ri" ((short) pos));
}

static __inline int_ _dos_hendspso (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #8,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_hendspsp (int_ pos, const char *mesptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #8,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) mesptr), "ri" ((short) pos));

    return reg_d0;
}

static __inline int_ _dos_hendspsr (int_ pos, const char *mesptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #10,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) mesptr), "ri" ((short) pos));

    return reg_d0;
}

static __inline void _dos_hendspsc (void_)
{
    asm volatile ("move.w #11,-(sp)\n\t"
		  "dc.w HENDSP\n\t"
		  "addq.w #2,sp"
		  :/* no outputs */
		  :/* no inputs */);
}

static __inline int_ _dos_curdrv (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w CURDRV"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_getss (struct _inpptr *inpptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "dc.w GETSS\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0), "=m" (*inpptr)
		  :"g" ((int_) inpptr));

    return reg_d0;
}

static __inline int_ _dos_fgetc (int_ fno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w FGETC\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_fgets (struct _inpptr *inpptr, int_ fno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w FGETS\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0), "=m" (*inpptr)
		  :"ri" ((short) fno), "g" ((int_) inpptr));

    return reg_d0;
}

static __inline void _dos_fputc (int_ code, int_ fno)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "move.w %1,-(sp)\n\t"
		  "dc.w FPUTC\n\t"
		  "addq.w #4,sp"
		  :/* no outputs */
		  :"ri" ((short) fno), "ri" ((short) code)
		  :"d0");
}

static __inline void _dos_fputs (const char *mesptr, int_ fno)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "dc.w FPUTS\n\t"
		  "addq.w #6,sp"
		  :/* no outputs */
		  :"ri" ((short) fno), "g" ((int_) mesptr)
		  :"d0");
}

static __inline void _dos_allclose (void_)
{
    asm volatile ("dc.w ALLCLOSE"
		  :/* no outputs */
		  :/* no inputs */);
}

static __inline int_ _dos_super (int_ stack)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l sp,a1\n\t"
		  "move.l %1,-(sp)\n\t"
		  "dc.w SUPER\n\t"
		  "move.l a1,sp\n\t"
		  :"=d" (reg_d0)
		  :"g" ((int_) stack)
		  :"d0", "a1");

    return reg_d0;
}

static __inline void _dos_fnckeygt (int_ fno, char *buffer)
{
    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "dc.w FNCKEY\n\t"
		  "addq.w #6,sp"
		  :"=m" (*buffer)
		  :"g" ((int_) buffer), "ri" ((short) fno)
		  :"d0");
}

static __inline void _dos_fnckeyst (int_ fno, const char *buffer)
{
    int_ mode = fno + 0x100;

    asm volatile ("move.l %0,-(sp)\n\t"
		  "move.w %1,-(sp)\n\t"
		  "dc.w FNCKEY\n\t"
		  "addq.w #6,sp"
		  :/* no outputs */
		  :"g" ((int_) buffer), "ri" ((short) mode)
		  :"d0");
}

static __inline int_ _dos_c_putc (int_ code)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "clr.w -(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) code));

    return reg_d0;
}

static __inline int_ _dos_c_print_ (const char *msgptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w #1,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) msgptr));

    return reg_d0;
}

static __inline int_ _dos_c_color (int_ attr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #2,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) attr));

    return reg_d0;
}

static __inline int_ _dos_c_locate (int_ x, int_ y)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #3,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) y), "ri" ((short) x));

    return reg_d0;
}

static __inline int_ _dos_c_down_s (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #4,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_up_s (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #5,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_up (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #6,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline int_ _dos_c_down (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #7,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline int_ _dos_c_right (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #8,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline int_ _dos_c_left (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #9,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline int_ _dos_c_cls_ed (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("clr.w -(sp)\n\t"
		  "move.w #10,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_cls_st (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #1,-(sp)\n\t"
		  "move.w #10,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_cls_al (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #2,-(sp)\n\t"
		  "move.w #1,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_era_ed (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("clr.w -(sp)\n\t"
		  "move.w #11,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_era_st (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #1,-(sp)\n\t"
		  "move.w #11,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_era_al (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #2,-(sp)\n\t"
		  "move.w #11,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_ins (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #12,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline int_ _dos_c_del (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #13,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline int_ _dos_c_fnkmod (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #14,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline int_ _dos_c_window (int_ ys, int_ yl)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #15,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) yl), "ri" ((short) ys));

    return reg_d0;
}

static __inline int_ _dos_c_width (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #16,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline int_ _dos_c_curon (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #17,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_c_curoff (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #18,-(sp)\n\t"
		  "dc.w CONCTRL\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_k_keyinp (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("clr.w -(sp)\n\t"
		  "dc.w KEYCTRL\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_k_keysns (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #1,-(sp)\n\t"
		  "dc.w KEYCTRL\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_k_sftsns (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w #2,-(sp)\n\t"
		  "dc.w KEYCTRL\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_k_keybit (int_ n)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #3,-(sp)\n\t"
		  "dc.w KEYCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) n));

    return reg_d0;
}

static __inline void _dos_k_insmod (int_ n)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "move.w #4,-(sp)\n\t"
		  "dc.w KEYCTRL\n\t"
		  "addq.w #4,sp"
		  :/* no outputs */
		  :"ri" ((short) n)
		  :"d0");
}

static __inline void *_dos_intvcs (int_ intno, void *jobadr)
{
    register void *reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "dc.w INTVCS\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) jobadr), "ri" ((short) intno));

    return (void *) reg_d0;
}

static __inline void _dos_pspset (struct _psp *pspadr)
{
    asm volatile ("move.l %0,-(sp)\n\t"
		  "dc.w PSPSET\n\t"
		  "addq.w #4,sp"
		  :/* no outputs */
		  :"g" ((int_) pspadr));
}

static __inline int_ _dos_gettim2 (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w GETTIM2\n\t"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_settim2 (int_ time)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w SETTIM2\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) time));

    return reg_d0;
}

static __inline int_ _dos_namests (const char *file, struct _namestbuf *buffer)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w NAMESTS\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0), "=m" (*buffer)
		  :"g" ((int_) buffer), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_getdate (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w GETDATE\n\t"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_setdate (int_ date)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w SETDATE\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) date));

    return reg_d0;
}

static __inline int_ _dos_gettime (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w GETTIME\n\t"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_settime (int_ time)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w SETTIME\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) time));

    return reg_d0;
}

static __inline void _dos_verify (int_ flag)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "dc.w VERIFY\n\t"
		  "addq.w #2,sp"
		  :/* no outputs */
		  :"ri" ((short) flag));
}

static __inline int_ _dos_dup0 (int_ fno, int_ newno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "dc.w DUP0\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) newno), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_  __const _dos_vernum (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w VERNUM"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline void __volatile _dos_keeppr (int_ prglen, int_ code)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "dc.w KEEPPR"
		  :/* no outputs */
		  :"ri" ((short) code), "g" ((int_) prglen));
}

static __inline int_ _dos_getdpb (int_ drive, struct _dpbptr *dpbptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "dc.w GETDPB\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0), "=m" (*dpbptr)
		  :"g" ((int_) dpbptr), "ri" ((short) drive));

    return reg_d0;
}

static __inline int_ _dos_breakck (int_ flag)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w BREAKCK\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) flag));

    return reg_d0;
}

static __inline void _dos_drvxchg (int_ old, int_ new)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "move.w %1,-(sp)\n\t"
		  "dc.w DRVXCHG\n\t"
		  "addq.w #4,sp"
		  :/* no outputs */
		  :"ri" ((short) new), "ri" ((short) old)
		  :"d0");
}

static __inline void *_dos_intvcg (int_ intno)
{
    register void *reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w INTVCG\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) intno));

    return (void *) reg_d0;
}

static __inline int_ _dos_dskfre (int_ drive, struct _freeinf *buffer)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "dc.w DSKFRE\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0), "=m" (*buffer)
		  :"g" ((int_) buffer), "ri" ((short) drive));

    return reg_d0;
}

static __inline int_ _dos_nameck (const char *file, struct _nameckbuf *buffer)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w NAMECK\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0), "=m" (*buffer)
		  :"g" ((int_) buffer), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_mkdir (const char *file)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w MKDIR\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_rmdir (const char *file)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w RMDIR\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_chdir (const char *file)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w CHDIR\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_create (const char *file, int_ attr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "dc.w CREATE\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) attr), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_open (const char *file, int_ mode)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "dc.w OPEN\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) mode), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_close (int_ fno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w CLOSE\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_read (int_ fno, char *buffer, int_ size)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.w %4,-(sp)\n\t"
		  "dc.w READ\n\t"
		  "lea.l 10(sp),sp"
		  :"=d" (reg_d0), "=m" (*buffer)
		  :"g" ((int_) size), "g" ((int_) buffer), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_write (int_ fno, const char *buffer, int_ size)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "dc.w WRITE\n\t"
		  "lea.l 10(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) size), "g" ((int_) buffer), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_delete (const char *file)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w DELETE\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) file));

    return reg_d0;
}

static __inline long _dos_seek (int_ fno, int_ offset, int_ mode)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "dc.w SEEK\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) mode), "g" ((int_) offset), "ri" ((short) fno));

    return reg_d0;
}

static __inline dosmode_t _dos_chmod (const char *file, int_ attr)
{
    register dosmode_t reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "dc.w CHMOD\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) attr), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_ioctrlgt (int_ fno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "clr.w -(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_ioctrlst (int_ fno, int_ data)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #1,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) data), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_ioctrlrh (int_ fno, char *buffer, int_ size)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.w #2,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) size), "g" ((int_) buffer), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_ioctrlwh (int_ fno, const char *buffer, int_ size)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.w #3,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) size), "g" ((int_) buffer), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_ioctrlrd (int_ drive, char *buffer, int_ size)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.w %4,-(sp)\n\t"
		  "move.w #4,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0), "=m" (*buffer)
		  :"g" ((int_) size), "g" ((int_) buffer), "ri" ((short) drive));

    return reg_d0;
}

static __inline int_ _dos_ioctrlwd (int_ drive, const char *buffer, int_ size)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.w #5,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) size), "g" ((int_) buffer), "ri" ((short) drive));

    return reg_d0;
}

static __inline int_ _dos_ioctrlis (int_ fno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #6,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_ioctrlos (int_ fno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #7,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_ioctrldvgt (int_ drive)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #9,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) drive));

    return reg_d0;
}

static __inline int_ _dos_ioctrlfdgt (int_ fno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w #10,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_ioctrlrtset (int_ count, int_ time)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w #11,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) time), "ri" ((short) count));

    return reg_d0;
}

static __inline int_ _dos_ioctrldvctl (int_ drive, int_ f_code, char *buff)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.w #12,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "lea.l 10(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) buff), "ri" ((short) f_code), "ri" ((short) drive));

    return reg_d0;
}

static __inline int_ _dos_ioctrlfdctl (int_ fno, int_ f_code, char *buff)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.w #13,-(sp)\n\t"
		  "dc.w IOCTRL\n\t"
		  "lea.l 10(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) buff), "ri" ((short) f_code), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_dup (int_ fno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w DUP\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_dup2 (int_ fno, int_ newno)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "dc.w DUP2\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) newno), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_curdir (int_ drive, char *buffer)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "dc.w CURDIR\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) buffer), "ri" ((short) drive));

    return reg_d0;
}

static __inline void *_dos_malloc (int_ size)
{
    register void *reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w MALLOC\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) size));

    return (void *) reg_d0;
}

static __inline int_ _dos_mfree (void *memptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w MFREE\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) memptr));

    return reg_d0;
}

static __inline int_ _dos_setblock (void *memptr, int_ newlen)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "dc.w SETBLOCK\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) newlen), "g" ((int_) memptr));

    return reg_d0;
}

static __inline int_ _dos_exec2 (int_ md, const char *file,
				 const char *p1, const char *p2)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("movem.l d3-d7/a3-a6,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.w %4,-(sp)\n\t"
		  "dc.w EXEC\n\t"
		  "lea.l 14(sp),sp\n\t"
		  "movem.l (sp)+,d3-d7/a3-a6"
		  :"=d" (reg_d0)
		  :"g" ((int_) p2), "g" ((int_) p1), "g" ((int_) file), "ri" ((short) md)
		  :"d1", "d2", "a0", "a1", "a2");

    return reg_d0;
}

static __inline int_ _dos_loadexec (const char *file,
				    const struct _comline *comline, const char *envptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("movem.l d3-d7/a3-a6,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "clr.w -(sp)\n\t"
		  "dc.w EXEC\n\t"
		  "lea.l 14(sp),sp\n\t"
		  "movem.l (sp)+,d3-d7/a3-a6"
		  :"=d" (reg_d0)
		  :"g" ((int_) envptr), "g" ((int_) comline), "g" ((int_) file)
		  :"d1", "d2", "a0", "a1", "a2");

    return reg_d0;
}

static __inline int_ _dos_load (const char *file,
				const struct _comline *comline, const char *envptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("movem.l d3-d7/a3-a6,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.w #1,-(sp)\n\t"
		  "dc.w EXEC\n\t"
		  "lea.l 14(sp),sp\n\t"
		  "movem.l (sp)+,d3-d7/a3-a6"
		  :"=d" (reg_d0)
		  :"g" ((int_) envptr), "g" ((int_) comline), "g" ((int_) file)
		  :"d1", "d2", "a0", "a1", "a2");

    return reg_d0;
}

static __inline int_ _dos_pathchk (const char *file,
				   const struct _comline *comline, const char *envp)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.w #2,-(sp)\n\t"
		  "dc.w EXEC\n\t"
		  "lea.l 14(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) envp), "g" ((int_) comline), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_loadonly (const char *file,
				    const void *loadadr, const void *limitadr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.w #3,-(sp)\n\t"
		  "dc.w EXEC\n\t"
		  "lea.l 14(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) limitadr), "g" ((int_) loadadr), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_execonly (void *jmpaddr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("movem.l d3-d7/a3-a6,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "move.w #4,-(sp)\n\t"
		  "dc.w EXEC\n\t"
		  "addq.w #6,sp\n\t"
		  "movem.l (sp)+,d3-d7/a3-a6"
		  :"=d" (reg_d0)
		  :"g" ((int_) jmpaddr)
		  :"d1", "d2", "a0", "a1", "a2");

    return reg_d0;
}

static __inline int_ _dos_bindno (const char *file1, const char *file2)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("movem.l d3-d7/a3-a6,-(sp)\n\t"
		  "clr.l -(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w #5,-(sp)\n\t"
		  "dc.w EXEC\n\t"
		  "lea.l 14(sp),sp\n\t"
		  "movem.l (sp)+,d3-d7/a3-a6"
		  :"=d" (reg_d0)
		  :"g" ((int_) file2), "g" ((int_) file1)
		  :"d1", "d2", "a0", "a1", "a2");

    return reg_d0;
}

static __inline void __volatile _dos_exit2 (int_ code)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "dc.w EXIT2\n\t"
		  :/* no outputs */
		  :"ri" ((short) code));
}

static __inline int_ _dos_wait (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w WAIT\n\t"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_files (struct _filbuf *buffer,
				 const char *file, int_ attr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.l %4,-(sp)\n\t"
		  "dc.w FILES\n\t"
		  "lea.l 10(sp),sp"
		  :"=d" (reg_d0), "=m" (*buffer)
		  :"ri" ((short) attr), "g" ((int_) file), "g" ((int_) buffer));

    return reg_d0;
}

static __inline int_ _dos_nfiles (struct _filbuf *buffer)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "dc.w NFILES\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0), "=m" (*buffer)
		  :"g" ((int_) buffer));

    return reg_d0;
}

static __inline struct _psp *_dos_setpdb (struct _psp *pdbadr)
{
    register struct _psp *reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w SETPDB\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) pdbadr));

    return (struct _psp *) reg_d0;
}

static __inline struct _psp *_dos_getpdb (void_)
{
    register struct _psp *reg_d0 asm ("d0");

    asm volatile ("dc.w GETPDB"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return (struct _psp *) reg_d0;
}

static __inline int_ _dos_setenv (const char *name, const char *env, const char *buffer)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w SETENV\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) buffer), "g" ((int_) env), "g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_getenv (const char *name, const char *env, char *buffer)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w GETENV\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) buffer), "g" ((int_) env), "g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_verifyg (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w VERIFYG"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_common_ck (const char *name)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "clr.w -(sp)\n\t"
		  "dc.w COMMON\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_common_rd (const char *name,
				     int_ pos, char *buffer, int_ len)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.l %4,-(sp)\n\t"
		  "move.l %5,-(sp)\n\t"
		  "move.w #1,-(sp)\n\t"
		  "dc.w COMMON\n\t"
		  "lea.l 18(sp),sp"
		  :"=d" (reg_d0), "=m" (*buffer)
		  :"g" ((int_) len), "g" ((int_) buffer), "g" ((int_) pos), "g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_common_wt (const char *name,
				     int_ pos, const char *buffer, int_ len)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.l %4,-(sp)\n\t"
		  "move.w #2,-(sp)\n\t"
		  "dc.w COMMON\n\t"
		  "lea.l 18(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) len), "g" ((int_) buffer), "g" ((int_) pos), "g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_common_lk (const char *name,
				     int_ pos, int_ id_psp, int_ len)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.l %4,-(sp)\n\t"
		  "move.w #3,-(sp)\n\t"
		  "dc.w COMMON\n\t"
		  "lea.l 18(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) len), "g" ((int_) id_psp), "g" ((int_) pos), "g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_common_fre (const char *name,
				      int_ pos, int_ id_psp, int_ len)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.l %4,-(sp)\n\t"
		  "move.w #4,-(sp)\n\t"
		  "dc.w COMMON\n\t"
		  "lea.l 18(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) len), "g" ((int_) id_psp), "g" ((int_) pos), "g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_common_del (const char *name)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w #5,-(sp)\n\t"
		  "dc.w COMMON\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_move (const char *old, const char *new)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "dc.w RENAME\n\t"
		  "addq.w #8,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) new), "g" ((int_) old));

    return reg_d0;
}

static __inline int_ _dos_rename (const char *old, const char *new)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "dc.w RENAME\n\t"
		  "addq.w #8,sp"
		  : "=d" (reg_d0)
		  : "g" ((int_) new), "g" ((int_) old));

    return reg_d0;
}

static __inline int_ _dos_filedate (int_ fno, int_ datetime)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "dc.w FILEDATE\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) datetime), "ri" ((short) fno));

    return reg_d0;
}

static __inline void *_dos_malloc2 (int_ md, int_ size)
{
    register void *reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "dc.w MALLOC2\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) size), "ri" ((short) md));

    return (void *) reg_d0;
}

static __inline int_ _dos_maketmp (const char *file, int_ attr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "dc.w MAKETMP\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) attr), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_newfile (const char *file, int_ attr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "dc.w NEWFILE\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) attr), "g" ((int_) file));

    return reg_d0;
}

static __inline int_ _dos_lock (int_ fno, int_ offset, int_ len)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "clr.w -(sp)\n\t"
		  "dc.w LOCK\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) len), "g" ((int_) offset), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_unlock (int_ fno, int_ offset, int_ len)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.w #1,-(sp)\n\t"
		  "dc.w LOCK\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) len), "g" ((int_) offset), "ri" ((short) fno));

    return reg_d0;
}

static __inline int_ _dos_getassign (const char *path1, char *path2)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "clr.w -(sp)\n\t"
		  "dc.w ASSIGN\n\t"
		  "lea.l 10(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) path2), "g" ((int_) path1));

    return reg_d0;
}

static __inline int_ _dos_makeassign (const char *path1,
				      const char *path2, int_ mode)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.w #1,-(sp)\n\t"
		  "dc.w ASSIGN\n\t"
		  "lea.l 12(sp),sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) mode), "g" ((int_) path2), "g" ((int_) path1));

    return reg_d0;
}

static __inline int_ _dos_rassign (const char *path1)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w #4,-(sp)\n\t"
		  "dc.w ASSIGN\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) path1));

    return reg_d0;
}

static __inline union _fcb *_dos_getfcb (unsigned int_ fno)
{
    register union _fcb *reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w GETFCB\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"g" ((short) fno));

    return (union _fcb *) reg_d0;
}

static __inline void *_dos_s_malloc (int_ md, int_ size)
{
    register void *reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "dc.w S_MALLOC\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) size), "ri" ((short) md));

    return (void *) reg_d0;
}

static __inline int_ _dos_s_mfree (void *memptr)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w S_MFREE\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) memptr));

    return reg_d0;
}

static __inline int_ _dos_s_process (int_ id, int_ start, int_ size, int_ i_len)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "move.w %4,-(sp)\n\t"
		  "dc.w S_PROCESS\n\t"
		  "lea.l 14(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) i_len), "g" ((int_) size), "g" ((int_) start), "ri" ((short) id));

    return reg_d0;
}

static __inline void __volatile _dos_retshell (void_)
{
    asm volatile ("dc.w RETSHELL"
		  :/* no outputs */
		  :/* no inputs */);
}

static __inline void __volatile _dos_ctlabort (void_)
{
    asm volatile ("dc.w CTRABORT"
		  :/* no outputs */
		  :/* no inputs */);
}

static __inline void __volatile _dos_errabort (void_)
{
    asm volatile ("dc.w ERRABORT"
		  :/* no outputs */
		  :/* no inputs */);
}

static __inline void _dos_diskred (void *addr,
				   int_ drive, int_ sector, int_ seclen)
{
    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.l %4,-(sp)\n\t"
		  "dc.w DISKRED\n\t"
		  "lea.l 10(sp),sp"
		  :"=m" (*(char *) addr)
		  :"ri" ((short) seclen), "ri" ((short) sector),
		   "ri" ((short) drive), "g" ((int_) addr)
		  :"d0");
}

static __inline void _dos_diskred2 (void *addr,
				    int_ drive, int_ sector, int_ seclen)
{
    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "bset.l #31,%4\n\t"
		  "move.l %4,-(sp)\n\t"
		  "dc.w DISKRED\n\t"
		  "lea.l 14(sp),sp"
		  :"=m" (*(char *) addr)
		  :"ri" ((int_) seclen), "ri" ((int_) sector),
		   "ri" ((short) drive), "dm" ((int_) addr));
}

static __inline void _dos_diskwrt (const void *addr,
				   int_ drive, int_ sector, int_ seclen)
{
    asm volatile ("move.w %0,-(sp)\n\t"
		  "move.w %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w DISKWRT\n\t"
		  "lea.l 10(sp),sp"
		  :/* no outputs */
		  :"ri" ((short) seclen), "ri" ((short) sector), "ri" ((short) drive), "g" ((int_) addr)
		  :"d0");
}

static __inline void _dos_diskwrt2 (const void *addr,
				    int_ drive, int_ sector, int_ seclen)
{
    asm volatile ("move.l %0,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "bset.l #31,%3\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w DISKWRT\n\t"
		  "lea.l 14(sp),sp"
		  :/* no outputs */
		  :"ri" ((int_) seclen), "ri" ((int_) sector),
		   "ri" ((short) drive), "dm" ((int_) addr));
}

static __inline struct _indos *_dos_indosflg (void_)
{
    register struct _indos *reg_d0 asm ("d0");

    asm volatile ("dc.w INDOSFLG\n\t"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline void _dos_super_jsr (void (*func) (),
				     struct _dregs *inregs, struct _dregs *outregs)
{
    asm volatile ("movem.l d0-d7/a0-a6,-(sp)\n\t"
		  "move.l %0,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "move.l %2,a0\n\t"
		  "movem.l (a0),d0-d7/a0-a6\n\t"
		  "dc.w SUPER_JSR\n\t"
		  "move.l a6,-(sp)\n\t"
		  "move.l 8(sp),a6\n\t"
		  "move.l (sp)+,56(a6)\n\t"
		  "addq.w #8,sp\n\t"
		  "movem.l d0-d7/a0-a5,(a6)\n\t"
		  "movem.l (sp)+,d0-d7/a0-a6"
		  :/* no outputs */
		  :"g" ((int_) outregs), "g" ((int_) func), "g" ((int_) inregs));
}

static __inline int_ _dos_memcpy (void *s_addr, void *d_addr, int_ mode)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w MEMCPY\n\t"
		  "lea.l 10(sp),sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) mode), "g" ((int_) d_addr), "g" ((int_) s_addr));

    return reg_d0;
}

static __inline int_ _dos_open_pr (const char *name, int_ counter,
				   int_ usp, int_ ssp,
				   int_ sr, int_ pc,
				   struct _prcctrl *buff, long sleep_time)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %0,-(sp)\n\t"
		  "move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.l %4,-(sp)"
		  :/* no outputs */
		  :"g" ((int_) sleep_time), "g" ((int_) buff), "g" ((int_) pc), "ri" ((short) sr),
		   "g" ((int_) ssp));

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.w %2,-(sp)\n\t"
		  "move.l %3,-(sp)\n\t"
		  "dc.w OPEN_PR\n\t"
		  "lea.l  28(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) usp), "ri" ((short) counter), "g" ((int_) name));

    return reg_d0;
}

static __inline int_ _dos_kill_pr (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w KILL_PR"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline int_ _dos_get_pr (int_ id, struct _prcptr *buff)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "dc.w GET_PR\n\t"
		  "addq.w #6,sp"
		  :"=d" (reg_d0), "=m" (*buff)
		  :"g" ((int_) buff), "ri" ((short) id));

    return reg_d0;
}

static __inline int_ _dos_suspend_pr (int_ id)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.w %1,-(sp)\n\t"
		  "dc.w SUSPEND_PR\n\t"
		  "addq.w #2,sp"
		  :"=d" (reg_d0)
		  :"ri" ((short) id));

    return reg_d0;
}

static __inline int_ _dos_sleep_pr (long time)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "dc.w SLEEP_PR\n\t"
		  "addq.w #4,sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) time));

    return reg_d0;
}

static __inline int_ _dos_send_pr (int_ my_id, int_ your_id,
				   int_ command, char *buff, long length)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("move.l %1,-(sp)\n\t"
		  "move.l %2,-(sp)\n\t"
		  "move.w %3,-(sp)\n\t"
		  "move.w %4,-(sp)\n\t"
		  "move.w %5,-(sp)\n\t"
		  "dc.w SEND_PR\n\t"
		  "lea.l 14(sp),sp"
		  :"=d" (reg_d0)
		  :"g" ((int_) length), "g" ((int_) buff), "ri" ((short) command),
		   "ri" ((short) your_id), "ri" ((short) my_id));

    return reg_d0;
}

static __inline long _dos_time_pr (void_)
{
    register int_ reg_d0 asm ("d0");

    asm volatile ("dc.w TIME_PR\n\t"
		  :"=d" (reg_d0)
		  :/* no inputs */);

    return reg_d0;
}

static __inline void _dos_change_pr (void_)
{
    asm volatile ("dc.w CHANGE_PR"
		  :/* no outputs */
		  :/* no inputs */);
}

#endif
