/*
 * PROJECT C Library, X68000 PROGRAMMING INTERFACE DEFINITION
 * --------------------------------------------------------------------
 * This file is written by the Project C Library Group,  and completely
 * in public domain. You can freely use, copy, modify, and redistribute
 * the whole contents, without this notice.
 * --------------------------------------------------------------------
 * $Id: wctype.h,v 1.2 1993/04/30 19:10:54 mura Exp $
 */

#ifndef __wctype_h__
#define __wctype_h__

#ifndef __cdecl_h__
#include <cdecl.h>
#endif

#ifndef __ctype_h__
#include <ctype.h>
#endif

#ifndef __wchar_t_defined__
#define __wchar_t_defined__
typedef int_ wchar_t;
#endif

#ifndef __wint_t_defined__
#define __wint_t_defined__
typedef int_ wint_t;
#endif

typedef struct {
    wchar_t tmin;	  /* minimum type code */
    wchar_t tmax;	  /* maximum type code */
    unsigned char *index; /* class index */
    unsigned int_ *type;  /* class type */
    wchar_t cmin;	  /* minimum conversion code */
    wchar_t cmax;	  /* maximum conversion code */
    wchar_t *code;	  /* conversion code */
} wctype_t;

#define _ISASCII(x) (((x) & ~0x7f) == 0)
#define _ISISO(x)   (((x) & ~0xff) == 0)

EXT (int_ __const iswalnum (wint_t));
EXT (int_ __const iswalpha (wint_t));
EXT (int_ __const iswascii (wint_t));
EXT (int_ __const iswcntrl (wint_t));
EXT (int_ __const iswdigit (wint_t));
EXT (int_ __const iswgraph (wint_t));
EXT (int_ __const iswlower (wint_t));
EXT (int_ __const iswprint (wint_t));
EXT (int_ __const iswpunct (wint_t));
EXT (int_ __const iswspace (wint_t));
EXT (int_ __const iswupper (wint_t));
EXT (int_ __const iswxdigit (wint_t));
EXT (int_ iswctype (wint_t, wctype_t));
EXT (wctype_t wctype (const char *));
EXT (wint_t __const towlower (wint_t));
EXT (wint_t __const towupper (wint_t));

#ifndef __NO_CTYPE_INLINE__

#define iswalnum(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISALNUM) : 0)
#define iswalpha(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISALPHA) : 0)
#define iswcntrl(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISCNTRL) : 0)
#define iswdigit(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISDIGIT) : 0)
#define iswgraph(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISGRAPH) : 0)
#define iswlower(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISLOWER) : 0)
#define iswprint(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISPRINT) : 0)
#define iswpunct(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISPUNCT) : 0)
#define iswspace(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISSPACE) : 0)
#define iswupper(c)  ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISUPPER) : 0)
#define iswxdigit(c) ((int_) _ISASCII (c) ? (_ctype[(int_) (c)] & _ISXDIGT) : 0)
#define iswascii(c)  ((int_) _ISASCII (c))
#define towlower(c)  ((int_) _ISASCII (c) ? _ctype_lower[(int_) (c)] : (c))
#define towupper(c)  ((int_) _ISASCII (c) ? _ctype_upper[(int_) (c)] : (c))

#endif

#endif
